@@ -5,7 +5,7 @@ use warnings;
 
 use parent qw(Ryu::Node);
 
-our $VERSION = '0.033'; # VERSION
+our $VERSION = '0.034'; # VERSION
 
 =head1 NAME
 
@@ -1342,7 +1342,7 @@ sub some {
         return if $src->completed->is_ready;
         return unless $code->($_);
         $src->emit(1);
-        $src->completed->done 
+        $src->completed->done
     });
     $src
 }
@@ -1361,13 +1361,13 @@ sub every {
     $self->completed->on_done(sub {
         return if $src->completed->is_ready;
         $src->emit(1);
-        $src->completed->done 
+        $src->completed->done
     });
     $self->each(sub {
         return if $src->completed->is_ready;
         return if $code->($_);
         $src->emit(0);
-        $src->completed->done 
+        $src->completed->done
     });
     $src
 }
@@ -1763,6 +1763,7 @@ Block until this source finishes.
 
 sub await {
     my ($self) = @_;
+    $self->prepare_await;
     my $f = $self->completed;
     $f->await until $f->is_ready;
     $self
@@ -1808,7 +1809,6 @@ sub get {
     if(my $parent = $self->parent) {
         $parent->await
     }
-    (delete $self->{on_get})->() if $self->{on_get};
     $f->transform(done => sub {
         @rslt
     })->get
@@ -1820,6 +1820,25 @@ for my $k (qw(then cancel fail on_ready transform is_ready is_done failure is_ca
 
 =head1 METHODS - Internal
 
+=head2 prepare_await
+
+Run any pre-completion callbacks (recursively) before
+we go into an await cycle.
+
+Used for compatibility with sync bridges when there's
+no real async event loop available.
+
+=cut
+
+sub prepare_await {
+    my ($self) = @_;
+    (delete $self->{on_get})->() if $self->{on_get};
+    return unless my $parent = $self->parent;
+    my $code = $parent->can('prepare_await');
+    local @_ = ($parent);
+    goto &$code;
+}
+
 =head2 chained
 
 Returns a new L<Ryu::Source> chained from this one.
@@ -1944,5 +1963,5 @@ Tom Molesworth <TEAM@cpan.org>
 
 =head1 LICENSE
 
-Copyright Tom Molesworth 2011-2018. Licensed under the same terms as Perl itself.
+Copyright Tom Molesworth 2011-2019. Licensed under the same terms as Perl itself.
 
